@@ -19,8 +19,6 @@ from dvc.parsing.interpolate import (
     recurse,
     str_interpolate,
 )
-from dvc.path_info import PathInfo
-from dvc.utils import relpath
 
 logger = logging.getLogger(__name__)
 SeqOrMap = Union[Sequence, Mapping]
@@ -356,24 +354,23 @@ class Context(CtxDict):
 
     @classmethod
     def load_from(
-        cls, fs, path: PathInfo, select_keys: List[str] = None
+        cls, fs, path: str, select_keys: List[str] = None
     ) -> "Context":
         from dvc.utils.serialize import LOADERS
 
-        file = relpath(path)
         if not fs.exists(path):
-            raise ParamsLoadError(f"'{file}' does not exist")
+            raise ParamsLoadError(f"'{path}' does not exist")
         if fs.isdir(path):
-            raise ParamsLoadError(f"'{file}' is a directory")
+            raise ParamsLoadError(f"'{path}' is a directory")
 
-        _, ext = os.path.splitext(file)
+        _, ext = os.path.splitext(path)
         loader = LOADERS[ext]
 
         data = loader(path, fs=fs)
         if not isinstance(data, Mapping):
             typ = type(data).__name__
             raise ParamsLoadError(
-                f"expected a dictionary, got '{typ}' in file '{file}'"
+                f"expected a dictionary, got '{typ}' in file '{path}'"
             )
 
         if select_keys:
@@ -382,12 +379,12 @@ class Context(CtxDict):
             except KeyError as exc:
                 key, *_ = exc.args
                 raise ParamsLoadError(
-                    f"could not find '{key}' in '{file}'"
+                    f"could not find '{key}' in '{path}'"
                 ) from exc
 
-        meta = Meta(source=file, local=False)
+        meta = Meta(source=path, local=False)
         ctx = cls(data, meta=meta)
-        ctx.imports[os.path.abspath(path)] = select_keys
+        ctx.imports[path] = select_keys
         return ctx
 
     def merge_update(self, other: "Context", overwrite=False):
@@ -396,31 +393,28 @@ class Context(CtxDict):
             raise ReservedKeyError(matches)
         return super().merge_update(other, overwrite=overwrite)
 
-    def merge_from(
-        self, fs, item: str, wdir: PathInfo, overwrite=False,
-    ):
+    def merge_from(self, fs, item: str, wdir: str, overwrite=False):
         path, _, keys_str = item.partition(":")
+        path = os.path.normpath(fs.path.join(wdir, path))
+
         select_keys = lfilter(bool, keys_str.split(",")) if keys_str else None
-
-        abspath = os.path.abspath(wdir / path)
-        path_info = PathInfo(abspath)
-        if abspath in self.imports:
-            if not select_keys and self.imports[abspath] is None:
+        if path in self.imports:
+            if not select_keys and self.imports[path] is None:
                 return  # allow specifying complete filepath multiple times
-            self.check_loaded(abspath, item, select_keys)
+            self.check_loaded(path, item, select_keys)
 
-        ctx = Context.load_from(fs, path_info, select_keys)
+        ctx = Context.load_from(fs, path, select_keys)
 
         try:
             self.merge_update(ctx, overwrite=overwrite)
         except ReservedKeyError as exc:
             raise ReservedKeyError(exc.keys, item) from exc
 
-        cp = ctx.imports[abspath]
-        if abspath not in self.imports:
-            self.imports[abspath] = cp
+        cp = ctx.imports[path]
+        if path not in self.imports:
+            self.imports[path] = cp
         elif cp:
-            self.imports[abspath].extend(cp)
+            self.imports[path].extend(cp)
 
     def check_loaded(self, path, item, keys):
         if not keys and isinstance(self.imports[path], list):
@@ -441,12 +435,12 @@ class Context(CtxDict):
         self,
         fs,
         vars_: List,
-        wdir: PathInfo,
+        wdir: str,
         stage_name: str = None,
         default: str = None,
     ):
         if default:
-            to_import = wdir / default
+            to_import = fs.path.join(wdir, default)
             if fs.exists(to_import):
                 self.merge_from(fs, default, wdir)
             else:
