@@ -18,7 +18,6 @@ if TYPE_CHECKING:
     from dvc.fs.base import FileSystem
     from dvc.objects.file import HashFile
     from dvc.repo.scm_context import SCMContext
-    from dvc.scm import Base
 
 logger = logging.getLogger(__name__)
 
@@ -57,17 +56,17 @@ class Repo:
     from dvc.repo.add import add
     from dvc.repo.checkout import checkout
     from dvc.repo.commit import commit
-    from dvc.repo.destroy import destroy
+    from dvc.repo.destroy import destroy  # type: ignore[misc]
     from dvc.repo.diff import diff
     from dvc.repo.fetch import fetch
-    from dvc.repo.freeze import freeze, unfreeze
+    from dvc.repo.freeze import freeze, unfreeze  # type: ignore[misc]
     from dvc.repo.gc import gc
-    from dvc.repo.get import get as _get
-    from dvc.repo.get_url import get_url as _get_url
-    from dvc.repo.imp import imp
+    from dvc.repo.get import get as _get  # type: ignore[misc]
+    from dvc.repo.get_url import get_url as _get_url  # type: ignore[misc]
+    from dvc.repo.imp import imp  # type: ignore[misc]
     from dvc.repo.imp_url import imp_url
-    from dvc.repo.install import install
-    from dvc.repo.ls import ls as _ls
+    from dvc.repo.install import install  # type: ignore[misc]
+    from dvc.repo.ls import ls as _ls  # type: ignore[misc]
     from dvc.repo.move import move
     from dvc.repo.pull import pull
     from dvc.repo.push import push
@@ -84,24 +83,15 @@ class Repo:
     def _get_repo_dirs(
         self,
         root_dir: str = None,
-        scm: "Base" = None,
-        rev: str = None,
+        fs: "FileSystem" = None,
         uninitialized: bool = False,
     ):
-        assert bool(scm) == bool(rev)
-
-        from dvc.fs.git import GitFileSystem
-        from dvc.scm import SCM, Base, Git, SCMError
+        from dvc.scm import SCM, Base, SCMError
         from dvc.utils.fs import makedirs
 
         dvc_dir = None
         tmp_dir = None
         try:
-            fs = (
-                GitFileSystem(scm=scm, rev=rev)
-                if isinstance(scm, Git) and rev
-                else None
-            )
             root_dir = self.find_root(root_dir, fs)
             dvc_dir = os.path.join(root_dir, self.DVC_DIR)
             tmp_dir = os.path.join(dvc_dir, "tmp")
@@ -151,7 +141,7 @@ class Repo:
     def __init__(
         self,
         root_dir=None,
-        scm=None,
+        fs=None,
         rev=None,
         subrepos=False,
         uninitialized=False,
@@ -160,39 +150,34 @@ class Repo:
         repo_factory=None,
     ):
         from dvc.config import Config
+        from dvc.data.db import ODBManager
         from dvc.data_cloud import DataCloud
         from dvc.fs.git import GitFileSystem
-        from dvc.fs.local import LocalFileSystem
+        from dvc.fs.local import localfs
         from dvc.lock import LockNoop, make_lock
-        from dvc.objects.db import ODBManager
-        from dvc.repo.live import Live
         from dvc.repo.metrics import Metrics
         from dvc.repo.params import Params
         from dvc.repo.plots import Plots
         from dvc.repo.stage import StageLoad
-        from dvc.scm import SCM, Git
+        from dvc.scm import SCM
         from dvc.stage.cache import StageCache
         from dvc.state import State, StateNoop
 
         self.url = url
         self._fs_conf = {"repo_factory": repo_factory}
+        self._fs = fs or localfs
+        self._scm = None
 
-        if rev and not scm:
-            scm = SCM(root_dir or os.curdir)
+        if rev and not fs:
+            self._scm = SCM(root_dir or os.curdir)
+            self._fs = GitFileSystem(scm=self._scm, rev=rev)
 
         self.root_dir, self.dvc_dir, self.tmp_dir = self._get_repo_dirs(
-            root_dir=root_dir, scm=scm, rev=rev, uninitialized=uninitialized
+            root_dir=root_dir, fs=self.fs, uninitialized=uninitialized
         )
 
-        if scm:
-            assert isinstance(scm, Git)
-            self._fs = GitFileSystem(scm=scm, rev=rev)
-        else:
-            self._fs = LocalFileSystem(url=self.root_dir)
-
         self.config = Config(self.dvc_dir, fs=self.fs, config=config)
         self._uninitialized = uninitialized
-        self._scm = scm
 
         # used by RepoFileSystem to determine if it should traverse subrepos
         self.subrepos = subrepos
@@ -200,7 +185,7 @@ class Repo:
         self.cloud = DataCloud(self)
         self.stage = StageLoad(self)
 
-        if scm or not self.dvc_dir:
+        if isinstance(self.fs, GitFileSystem) or not self.dvc_dir:
             self.lock = LockNoop()
             self.state = StateNoop()
             self.odb = ODBManager(self)
@@ -223,7 +208,6 @@ class Repo:
         self.metrics = Metrics(self)
         self.plots = Plots(self)
         self.params = Params(self)
-        self.live = Live(self)
 
         self.stage_collection_error_handler: Optional[
             Callable[[str, Exception], None]
@@ -326,29 +310,32 @@ class Repo:
 
     @classmethod
     def find_root(cls, root=None, fs=None) -> str:
-        root_dir = os.path.realpath(root or os.curdir)
+        from dvc.fs.local import LocalFileSystem, localfs
 
-        if fs:
-            if fs.isdir(os.path.join(root_dir, cls.DVC_DIR)):
-                return root_dir
-            raise NotDvcRepoError(f"'{root}' does not contain DVC directory")
+        root = root or os.curdir
+        root_dir = os.path.realpath(root)
+        fs = fs or localfs
 
-        if not os.path.isdir(root_dir):
+        if not fs.isdir(root_dir):
             raise NotDvcRepoError(f"directory '{root}' does not exist")
 
         while True:
-            dvc_dir = os.path.join(root_dir, cls.DVC_DIR)
-            if os.path.isdir(dvc_dir):
+            dvc_dir = fs.path.join(root_dir, cls.DVC_DIR)
+            if fs.isdir(dvc_dir):
                 return root_dir
-            if os.path.ismount(root_dir):
+            if isinstance(fs, LocalFileSystem) and os.path.ismount(root_dir):
                 break
-            root_dir = os.path.dirname(root_dir)
+            parent = fs.path.parent(root_dir)
+            if parent == root_dir:
+                break
+            root_dir = parent
 
-        message = (
-            "you are not inside of a DVC repository "
-            "(checked up to mount point '{}')"
-        ).format(root_dir)
-        raise NotDvcRepoError(message)
+        msg = "you are not inside of a DVC repository"
+
+        if isinstance(fs, LocalFileSystem):
+            msg = f"{msg} (checked up to mount point '{root_dir}')"
+
+        raise NotDvcRepoError(msg)
 
     @classmethod
     def find_dvc_dir(cls, root=None):
@@ -415,7 +402,7 @@ class Repo:
         def _add_suffix(objs: Set["HashFile"], suffix: str) -> None:
             from itertools import chain
 
-            from dvc.objects import iterobjs
+            from dvc.data import iterobjs
 
             for obj in chain.from_iterable(map(iterobjs, objs)):
                 if obj.name is not None:
@@ -493,7 +480,9 @@ class Repo:
     def repo_fs(self):
         from dvc.fs.repo import RepoFileSystem
 
-        return RepoFileSystem(self, subrepos=self.subrepos, **self._fs_conf)
+        return RepoFileSystem(
+            repo=self, subrepos=self.subrepos, **self._fs_conf
+        )
 
     @cached_property
     def index_db_dir(self):
@@ -504,8 +493,7 @@ class Repo:
         """Opens a specified resource as a file descriptor"""
         from dvc.fs.repo import RepoFileSystem
 
-        fs = RepoFileSystem(self, subrepos=True)
-        path = self.fs.path.join(self.root_dir, path)
+        fs = RepoFileSystem(repo=self, subrepos=True)
         try:
             with fs.open(
                 path, mode=mode, encoding=encoding, remote=remote
